@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *  Copyright (C) 2002-2007 Tomasz Kojm <tkojm@clamav.net>
  *
@@ -158,7 +158,7 @@ static void help(void)
     printf("\n");
     printf("                      Clam AntiVirus: Database Updater %s\n", get_version());
     printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-    printf("           (C) 2020 Cisco Systems, Inc.\n");
+    printf("           (C) 2021 Cisco Systems, Inc.\n");
     printf("\n");
     printf("    freshclam [options]\n");
     printf("\n");
@@ -184,7 +184,8 @@ static void help(void)
     printf("    --datadir=DIRECTORY                  Download new databases into DIRECTORY\n");
     printf("    --daemon-notify[=/path/clamd.conf]   Send RELOAD command to clamd\n");
     printf("    --local-address=IP   -a IP           Bind to IP for HTTP downloads\n");
-    printf("    --on-update-execute=COMMAND          Execute COMMAND after successful update\n");
+    printf("    --on-update-execute=COMMAND          Execute COMMAND after successful update.\n");
+    printf("                                         Use EXIT_1 to return 1 after successful database update.\n");
     printf("    --on-error-execute=COMMAND           Execute COMMAND if errors occurred\n");
     printf("    --on-outdated-execute=COMMAND        Execute COMMAND when software is outdated\n");
     printf("    --update-db=DBNAME                   Only update database DBNAME\n");
@@ -280,6 +281,14 @@ fc_error_t download_complete_callback(const char *dbFilename, void *context)
                 goto done;
             }
         } else {
+            /*
+             * Attempt to test database in a child process.
+             */
+
+            /* We need to be able to wait for the child process ourselves.
+             * We'll re-enable wait in the global handler when we're done. */
+            g_sigchildWait = 0;
+
             switch (pid = fork()) {
                 case -1: {
                     /*
@@ -391,6 +400,7 @@ done:
         logg("!Database test FAILED.\n");
     }
 
+    /* Re-enable the global handler's child process wait */
     g_sigchildWait = 1;
 
     return status;
@@ -1480,18 +1490,11 @@ fc_error_t perform_database_update(
     }
 
     if (0 < nTotalUpdated) {
-        if (NULL != notifyClamd)
+        if (NULL != notifyClamd) {
             notify(notifyClamd);
-
-        if (NULL != onUpdateExecute) {
-            execute("OnUpdateExecute", onUpdateExecute, bDaemonized);
         }
     }
 
-    if ((NULL != newVersion) && (NULL != onOutdatedExecute)) {
-        executeIfNewVersion(onOutdatedExecute, newVersion, bDaemonized);
-    }
-
     status = FC_SUCCESS;
 
 done:
@@ -1502,6 +1505,20 @@ done:
             cli_rmdirs(g_freshclamTempDirectory);
         }
     }
+
+    if (FC_SUCCESS == status) {
+        /* Run Execute commands after we clean up the temp directory,
+         * in case they want us to EXIT */
+        if (0 < nTotalUpdated) {
+            if (NULL != onUpdateExecute) {
+                execute("OnUpdateExecute", onUpdateExecute, bDaemonized);
+            }
+        }
+        if ((NULL != newVersion) && (NULL != onOutdatedExecute)) {
+            executeIfNewVersion(onOutdatedExecute, newVersion, bDaemonized);
+        }
+    }
+
     if (NULL != dnsUpdateInfo) {
         free(dnsUpdateInfo);
     }
